#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <caml/mlvalues.h>
#include <caml/fail.h>
#include <caml/alloc.h>
#include <caml/memory.h>
#include <caml/callback.h>

#include <locale.h>
#include <libintl.h>

static void init_locale()
{
    static int locale_is_set = 0;
    if (!locale_is_set) {
	setlocale(LC_ALL, "");
	locale_is_set = 1;
    }
}

value ocaml_simple_gettext_gettext(value msgid)
{
     CAMLparam1(msgid);
     init_locale();
     CAMLreturn(copy_string(gettext((const char *) String_val(msgid) )));
}

value ocaml_simple_gettext_dgettext(value domainname, value msgid)
{
     CAMLparam2(domainname,msgid);
     init_locale();
     CAMLreturn(copy_string(dgettext((const char *) String_val(domainname),
                                       (const char *) String_val(msgid)
                                       )));
}

value ocaml_simple_gettext_bindtextdomain(value domainname,value dirname)
{
     CAMLparam2(domainname,dirname);
     init_locale();
     if (!bindtextdomain((const char *) String_val(domainname),
			 (const char *) String_val(dirname)))
	 raise_sys_error(copy_string(strerror(errno)));
     CAMLreturn(Val_unit);
}

value ocaml_simple_gettext_textdomain(value domainname)
{
     CAMLparam1(domainname);
     init_locale();
     if (!textdomain((const char *) String_val(domainname)))
	 raise_sys_error(copy_string(strerror(errno)));
     CAMLreturn(Val_unit);
}
